package ink.wgink.module.instantmessage.service;

import ink.wgink.module.instantmessage.pojo.dtos.NoticeDTO;
import ink.wgink.module.instantmessage.pojo.pos.NoticePO;
import ink.wgink.module.instantmessage.pojo.vos.NoticeSendVO;
import ink.wgink.module.instantmessage.pojo.vos.NoticeVO;
import ink.wgink.module.instantmessage.websocket.pojo.body.CountNeedToDealWithBody;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: INoticeService
 * @Description: 消息通知
 * @Author: wanggeng
 * @Date: 2021/3/29 4:36 下午
 * @Version: 1.0
 */
public interface INoticeService {

    /**
     * 保存
     *
     * @param userDTO
     * @param notice  通知内容
     * @param isSend  是否发送
     */
    void save(UserDTO userDTO, NoticeVO.Notice notice, boolean isSend) throws Exception;

    /**
     * 更新已办状态
     *
     * @param userId
     * @param countNeedToDealWithBody
     */
    void updateHandleStatus(String userId, CountNeedToDealWithBody countNeedToDealWithBody);

    /**
     * 通知详情
     *
     * @param params
     * @return
     */
    NoticePO getPO(Map<String, Object> params);

    /**
     * 通知详情
     *
     * @param serviceId
     * @param userId
     * @return
     */
    NoticePO getPO(String serviceId, String userId);

    /**
     * 通知详情
     *
     * @param system
     * @param serviceId
     * @param userId
     * @return
     */
    NoticePO getPO(String system, String serviceId, String userId);

    /**
     * 通知详情
     *
     * @param system
     * @param module
     * @param serviceId
     * @param userId
     * @return
     */
    NoticePO getPO(String system, String module, String serviceId, String userId);

    /**
     * 通知详情
     *
     * @param system
     * @param module
     * @param menu
     * @param serviceId
     * @param userId
     * @return
     */
    NoticePO getPO(String system, String module, String menu, String serviceId, String userId);

    /**
     * 通知列表
     *
     * @param params
     * @return
     */
    List<NoticePO> listPO(Map<String, Object> params);

    /**
     * 通知列表
     *
     * @param serviceId
     * @param userId
     * @return
     */
    List<NoticePO> listPO(String serviceId, String userId);

    /**
     * 通知列表
     *
     * @param system
     * @param serviceId
     * @param userId
     * @return
     */
    List<NoticePO> listPO(String system, String serviceId, String userId);

    /**
     * 通知列表
     *
     * @param system
     * @param module
     * @param serviceId
     * @param userId
     * @return
     */
    List<NoticePO> listPO(String system, String module, String serviceId, String userId);

    /**
     * 通知列表
     *
     * @param system
     * @param module
     * @param menu
     * @param serviceId
     * @param userId
     * @return
     */
    List<NoticePO> listPO(String system, String module, String menu, String serviceId, String userId);

    /**
     * 通知数量
     *
     * @param userId
     * @param countNeedToDealWithBody
     * @return
     */
    List<CountNeedToDealWithBody> listNoticeCount(String userId, CountNeedToDealWithBody countNeedToDealWithBody);

    /**
     * 统计
     *
     * @param params
     * @return
     */
    Integer count(Map<String, Object> params);

    /**
     * 删除
     *
     * @param serviceIds
     */
    void deleteByServiceIds(List<String> serviceIds);

    /**
     * 删除通知
     *
     * @param noticeIds
     */
    void remove(List<String> noticeIds);

    /**
     * 通知列表
     *
     * @param serviceIds
     * @return
     */
    List<NoticePO> listPO(List<String> serviceIds);

    /**
     * 通知列表
     *
     * @param params
     * @return
     */
    List<NoticeDTO> list(Map<String, Object> params);

    /**
     * 通知分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<NoticeDTO>> listPage(ListPage page);

    /**
     * 发送通知
     *
     * @param noticeSendVO
     * @throws Exception
     */
    void notice(NoticeSendVO noticeSendVO) throws Exception;

    /**
     * 发送通知列表
     *
     * @param date
     * @return
     */
    List<NoticePO> listPOByDate(String date);

    /**
     * 发送通知列表
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<NoticePO> listPOByStartTimeAndEndTime(String startTime, String endTime);
}
