package ink.wgink.module.sms.service.sms;

import ink.wgink.interfaces.sms.ISmsBaseService;
import ink.wgink.module.sms.pojo.dtos.sms.SmsDTO;
import ink.wgink.module.sms.pojo.pos.sms.SmsPO;
import ink.wgink.module.sms.pojo.vos.sms.SmsSendVO;
import ink.wgink.module.sms.pojo.vos.sms.SmsVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: ISmsService
 * @Description: 短信
 * @Author: wanggeng
 * @Date: 2021/5/1 3:06 下午
 * @Version: 1.0
 */
public interface ISmsService extends ISmsBaseService {

    /**
     * 新增短信
     *
     * @param smsVO
     */
    void save(SmsVO smsVO);

    /**
     * 删除短信
     *
     * @param ids
     * @return
     */
    void remove(List<String> ids);

    /**
     * 短信详情
     *
     * @param params
     * @return
     */
    SmsDTO get(Map<String, Object> params);

    /**
     * 短信详情(通过ID)
     *
     * @param smsId
     * @return
     */
    SmsDTO get(String smsId);

    /**
     * 短信详情
     *
     * @param params
     * @return
     */
    SmsPO getPO(Map<String, Object> params);

    /**
     * 短信详情
     *
     * @param smsId
     * @return
     */
    SmsPO getPO(String smsId);

    /**
     * 短信列表
     *
     * @param params
     * @return
     */
    List<SmsDTO> list(Map<String, Object> params);

    /**
     * 短信分页列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<SmsDTO>> listPage(ListPage page);

    /**
     * 发送短信
     *
     * @param userId   用户ID
     * @param phone    用户手机
     * @param userName 用户昵称
     * @param content  发送内容
     */
    void sendContentByUserIdAndPhoneAndUserName(String userId, String phone, String userName, String content);

    /**
     * 发送验证码
     *
     * @param phone
     */
    void sendVerifyCode(String phone);

    /**
     * 发送短信
     *
     * @param smsSendVO
     */
    void send(SmsSendVO smsSendVO);

    /**
     * 发送短信列表
     *
     * @param params
     * @return
     */
    List<SmsPO> listPO(Map<String, Object> params);

    /**
     * 发送短信列表
     *
     * @param date
     * @return
     */
    List<SmsPO> listPOByDate(String date);

    /**
     * 发送短信个列表
     *
     * @param startTime
     * @param endTime
     * @return
     */
    List<SmsPO> listPOByStartTimeAndEndTime(String startTime, String endTime);
}
